@@ -1,20 +1,12 @@
 #!/usr/bin/python
-# This file is part of Ansible
-#
-# Ansible is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# Ansible is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
+# Copyright: Ansible Project
+# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-ANSIBLE_METADATA = {'metadata_version': '1.0',
+from __future__ import absolute_import, division, print_function
+__metaclass__ = type
+
+
+ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['preview'],
                     'supported_by': 'community'}
 
@@ -23,7 +15,7 @@ DOCUMENTATION = '''
 ---
 module: gce_instance_template
 version_added: "2.3"
-short_description: create or destroy intance templates of Compute Engine of GCP.
+short_description: create or destroy instance templates of Compute Engine of GCP.
 description:
      - Creates or destroy Google instance templates
        of Compute Engine of Google Cloud Platform.
@@ -87,7 +79,7 @@ options:
         If C(ephemeral), a new non-static address will be
         used.  If C(None), then no external address will
         be used.  To use an existing static IP address
-        specify adress name.
+        specify address name.
     default: "ephemeral"
   service_account_email:
     description:
@@ -140,6 +132,13 @@ options:
       - Support passing in the GCE-specific
         formatted networkInterfaces[] structure.
     default: null
+  disks_gce_struct:
+    description:
+      - Support passing in the GCE-specific
+        formatted formatted disks[] structure. Case sensitive.
+        see U(https://cloud.google.com/compute/docs/reference/latest/instanceTemplates#resource) for detailed information
+    default: null
+    version_added: "2.4"
   project_id:
     description:
       - your GCE project ID
@@ -153,6 +152,11 @@ options:
     description:
       - path to the JSON file associated with the service account email
     default: null
+  subnetwork_region:
+    version_added: "2.4"
+    description:
+      - Region that subnetwork resides in. (Required for subnetwork to successfully complete)
+    default: null
 requirements:
     - "python >= 2.6"
     - "apache-libcloud >= 0.13.3, >= 0.17.0 if using JSON credentials,
@@ -200,13 +204,43 @@ EXAMPLES = '''
         project_id: "{{ project_id }}"
         credentials_file: "{{ credentials_file }}"
         service_account_email: "{{ service_account_email }}"
+
+# Example playbook using disks_gce_struct
+- name: Compute Engine Instance Template Examples
+  hosts: localhost
+  vars:
+    service_account_email: "your-sa@your-project-name.iam.gserviceaccount.com"
+    credentials_file: "/path/to/your-key.json"
+    project_id: "your-project-name"
+  tasks:
+    - name: create instance template
+      gce_instance_template:
+        name: foo
+        size: n1-standard-1
+        state: present
+        project_id: "{{ project_id }}"
+        credentials_file: "{{ credentials_file }}"
+        service_account_email: "{{ service_account_email }}"
+        disks_gce_struct:
+          - device_name: /dev/sda
+            boot: true
+            autoDelete: true
+            initializeParams:
+              diskSizeGb: 30
+              diskType: pd-ssd
+              sourceImage: projects/debian-cloud/global/images/family/debian-8
+
 '''
 
 RETURN = '''
 '''
 
-from ansible.module_utils.basic import AnsibleModule
-from ansible.module_utils.gce import gce_connect
+import traceback
+try:
+    from ast import literal_eval
+    HAS_PYTHON26 = True
+except ImportError:
+    HAS_PYTHON26 = False
 
 try:
     import libcloud
@@ -220,16 +254,13 @@ try:
 except ImportError:
     HAS_LIBCLOUD = False
 
-try:
-    from ast import literal_eval
-    HAS_PYTHON26 = True
-except ImportError:
-    HAS_PYTHON26 = False
+from ansible.module_utils.basic import AnsibleModule
+from ansible.module_utils.gce import gce_connect
+from ansible.module_utils._text import to_native
 
 
 def get_info(inst):
     """Retrieves instance template information
-
     """
     return({
         'name': inst.name,
@@ -254,9 +285,9 @@ def create_instance_template(module, gce):
     disk_auto_delete = module.params.get('disk_auto_delete')
     network = module.params.get('network')
     subnetwork = module.params.get('subnetwork')
+    subnetwork_region = module.params.get('subnetwork_region')
     can_ip_forward = module.params.get('can_ip_forward')
     external_ip = module.params.get('external_ip')
-    service_account_email = module.params.get('service_account_email')
     service_account_permissions = module.params.get(
         'service_account_permissions')
     on_host_maintenance = module.params.get('on_host_maintenance')
@@ -265,8 +296,7 @@ def create_instance_template(module, gce):
     tags = module.params.get('tags')
     metadata = module.params.get('metadata')
     description = module.params.get('description')
-    disks = module.params.get('disks')
-
+    disks_gce_struct = module.params.get('disks_gce_struct')
     changed = False
 
     # args of ex_create_instancetemplate
@@ -314,7 +344,7 @@ def create_instance_template(module, gce):
     gce_args['network'] = gce_network
 
     if subnetwork is not None:
-        gce_args['subnetwork'] = subnetwork
+        gce_args['subnetwork'] = gce.ex_get_subnetwork(subnetwork, region=subnetwork_region)
 
     if can_ip_forward is not None:
         gce_args['can_ip_forward'] = can_ip_forward
@@ -355,6 +385,9 @@ def create_instance_template(module, gce):
     if tags is not None:
         gce_args['tags'] = tags
 
+    if disks_gce_struct is not None:
+        gce_args['disks_gce_struct'] = disks_gce_struct
+
     # Try to convert the user's metadata value into the format expected
     # by GCE.  First try to ensure user has proper quoting of a
     # dictionary-like syntax using 'literal_eval', then convert the python
@@ -426,8 +459,9 @@ def delete_instance_template(module, gce):
     try:
         instance = gce.ex_get_instancetemplate(name)
         current_state = "present"
-    except GoogleBaseError as err:
-        json_data = dict(msg='instance template not exists')
+    except GoogleBaseError as e:
+        json_data = dict(msg='instance template not exists: %s' % to_native(e),
+                         exception=traceback.format_exc())
 
     if current_state == "present":
         rc = instance.destroy()
@@ -478,26 +512,26 @@ def check_if_system_state_would_be_changed(module, gce):
     state = module.params.get("state")
     name = module.params.get("name")
 
-    instance = None
     try:
-        instance = gce.ex_get_instancetemplate(name)
+        gce.ex_get_instancetemplate(name)
         current_state = "present"
-    except GoogleBaseError as err:
-        module.fail_json(msg='GCE get instancetemplate problem')
+    except GoogleBaseError as e:
+        module.fail_json(msg='GCE get instancetemplate problem: %s' % to_native(e),
+                         exception=traceback.format_exc())
 
     if current_state != state:
         changed = True
 
     if current_state == "absent":
         if changed:
-            output = 'instance template {} will be created'.format(name)
+            output = 'instance template {0} will be created'.format(name)
         else:
-            output = 'nothing to do for instance template {} '.format(name)
+            output = 'nothing to do for instance template {0} '.format(name)
     if current_state == "present":
         if changed:
-            output = 'instance template {} will be detroyed'.format(name)
+            output = 'instance template {0} will be destroyed'.format(name)
         else:
-            output = 'nothing to do for instance template {} '.format(name)
+            output = 'nothing to do for instance template {0} '.format(name)
 
     return (changed, output)
 
@@ -529,6 +563,8 @@ def main():
             project_id=dict(),
             pem_file=dict(type='path'),
             credentials_file=dict(type='path'),
+            subnetwork_region=dict(),
+            disks_gce_struct=dict(type='list')
         ),
         mutually_exclusive=[['source', 'image']],
         required_one_of=[['image', 'image_family']],
@@ -544,8 +580,8 @@ def main():
 
     try:
         gce = gce_connect(module)
-    except GoogleBaseError as err:
-        module.fail_json(msg='GCE Connexion failed')
+    except GoogleBaseError as e:
+        module.fail_json(msg='GCE Connexion failed %s' % to_native(e), exception=traceback.format_exc())
 
     if module.check_mode:
         (changed, output) = check_if_system_state_would_be_changed(module, gce)
